@@ -22,7 +22,7 @@
 	This file incorporates work covered by the following copyright and
 	permission notice:
 
-	Copyright (c) 2013-2014, 2016-2017, 2020-2021 Cong Xu
+	Copyright (c) 2013-2014, 2016-2017, 2020-2021, 2023 Cong Xu
 	All rights reserved.
 
 	Redistribution and use in source and binary forms, with or without
@@ -288,7 +288,7 @@ static bool DidPlayerShoot(void)
 		continue;
 	}
 	const TActor *player = ActorGetByUID(p->ActorUID);
-	if (player->lastCmd & CMD_BUTTON1)
+	if (Button1(player->lastCmd))
 	{
 		return true;
 	}
@@ -349,8 +349,7 @@ int AICommand(const int ticks)
 		cmd = GetCmd(actor, delayModifier, rollLimit);
 		actor->aiContext->Delay = MAX(0, actor->aiContext->Delay - ticks);
 	}
-	CommandActor(actor, cmd, ticks);
-	actor->aiContext->LastCmd = cmd;
+	actor->aiContext->LastCmd = CommandActor(actor, cmd, ticks);
 	count++;
 	CA_FOREACH_END()
 	return count;
@@ -608,7 +607,7 @@ void AIAddRandomEnemies(const int enemies, const Mission *m)
 		const Character *c =
 			CArrayGet(&gCampaign.Setting.characters.OtherChars, charId);
 		GameEvent e = GameEventNewActorAdd(
-			PlaceAwayFromPlayers(&gMap, true, PLACEMENT_ACCESS_ANY), c, true);
+			PlaceAwayFromPlayers(&gMap, true, PLACEMENT_ACCESS_ANY), c, NULL);
 		e.u.ActorAdd.CharId = charId;
 		GameEventsEnqueue(&gGameEvents, e);
 		gBaddieCount++;
@@ -622,14 +621,14 @@ void InitializeBadGuys(void)
 	if (o->Type == OBJECTIVE_KILL &&
 		gMission.missionData->SpecialChars.size > 0)
 	{
-		const int charId =
-			CharacterStoreGetRandomSpecialId(&gCampaign.Setting.characters);
-		const Character *c =
-			CArrayGet(&gCampaign.Setting.characters.OtherChars, charId);
 		for (; o->placed < o->Count; o->placed++)
 		{
+			const int charId = CharacterStoreGetRandomSpecialId(
+				&gCampaign.Setting.characters);
+			const Character *c =
+				CArrayGet(&gCampaign.Setting.characters.OtherChars, charId);
 			GameEvent e = GameEventNewActorAdd(
-				PlaceAwayFromPlayers(&gMap, false, paFlags), c, true);
+				PlaceAwayFromPlayers(&gMap, false, paFlags), c, NULL);
 			e.u.ActorAdd.CharId = charId;
 			e.u.ActorAdd.ThingFlags = ObjectiveToThing(_ca_index);
 			GameEventsEnqueue(&gGameEvents, e);
@@ -650,7 +649,7 @@ void InitializeBadGuys(void)
 				MapHasLockedRooms(&gMap)
 					? PlacePrisoner(&gMap)
 					: PlaceAwayFromPlayers(&gMap, false, paFlags),
-				c, true);
+				c, NULL);
 			e.u.ActorAdd.CharId = charId;
 			e.u.ActorAdd.ThingFlags = ObjectiveToThing(_ca_index);
 			GameEventsEnqueue(&gGameEvents, e);
@@ -681,7 +680,7 @@ void CreateEnemies(void)
 		const Character *c =
 			CArrayGet(&gCampaign.Setting.characters.OtherChars, charId);
 		GameEvent e = GameEventNewActorAdd(
-			PlaceAwayFromPlayers(&gMap, true, PLACEMENT_ACCESS_ANY), c, true);
+			PlaceAwayFromPlayers(&gMap, true, PLACEMENT_ACCESS_ANY), c, NULL);
 		e.u.ActorAdd.CharId = charId;
 		GameEventsEnqueue(&gGameEvents, e);
 		gBaddieCount++;
