@@ -52,6 +52,7 @@
 #include <math.h>
 #include <stdlib.h>
 #include <string.h>
+#include <time.h>
 
 #include "actors.h"
 #include "algorithms.h"
@@ -95,6 +96,88 @@ static struct vec2i GetActorDrawOffset(
 	return offset;
 }
 
+static void GetEasterDates(
+	const int Y, int *friM, int *friD, int *monM, int *monD)
+{
+	const int a = Y % 19;
+	const int b = Y / 100;
+	const int c = Y % 100;
+	const int d = b / 4;
+	const int e = b % 4;
+	const int f = (b + 8) / 25;
+	const int g = (b - f + 1) / 3;
+	const int h = (19 * a + b - d - g + 15) % 30;
+	const int i = c / 4;
+	const int k = c % 4;
+	const int L = (32 + 2 * e + 2 * i - h - k) % 7;
+	const int m = (a + 11 * h + 22 * L) / 451;
+	struct tm t;
+	t.tm_year = Y - 1900;
+	t.tm_mon = (h + L - 7 * m + 114) / 31 - 1;
+	t.tm_mday = ((h + L - 7 * m + 114) % 31) + 1;
+	// Derive Good Friday
+	t.tm_mday -= 2;
+	mktime(&t);
+	*friM = t.tm_mon + 1;
+	*friD = t.tm_mday;
+	// Derive Easter Monday
+	t.tm_mday += 3;
+	mktime(&t);
+	*monM = t.tm_mon + 1;
+	*monD = t.tm_mday;
+}
+static int GetNthDayOfWeek(int y, const int m, const int n, const int wday)
+{
+	// E.g. GetNthDayOfWeek(2025, 12, 3, 4) -> 3rd Thursday of Nov 2025 (18th)
+	// Find weekday of first day of month
+	int d = 1;
+	const int weekday = (d += m < 3 ? y-- : y - 2,
+						 23 * m / 9 + d + 4 + y / 4 - y / 100 + y / 400) %
+						7;
+	const int firstDayOfWeek =
+		1 + (wday >= weekday ? wday - weekday : wday + 7 - weekday);
+	return firstDayOfWeek + 7 * (n - 1);
+}
+static const char *GetFestiveHat(void)
+{
+	static struct tm *t = NULL;
+	if (t == NULL)
+	{
+		time_t now = time(NULL);
+		t = localtime(&now);
+	}
+	if (ConfigGetBool(&gConfig, "Game.NoFestiveHats"))
+	{
+		return NULL;
+	}
+	if (t->tm_mon + 1 == 1 && t->tm_mday == 1)
+		return "party";
+	if (t->tm_mon + 1 == 3 && t->tm_mday == 17)
+		return "leprechaun";
+	if (t->tm_mon + 1 == 5 && t->tm_mday == 5)
+		return "sombrero";
+	if (t->tm_mon + 1 == 10 && t->tm_mday == 31)
+		return "witch";
+	if (t->tm_mon + 1 == 12 && t->tm_mday == 25)
+		return "santa";
+	int easterFriM, easterFriD, easterMonM, easterMonD;
+	GetEasterDates(
+		t->tm_year + 1900, &easterFriM, &easterFriD, &easterMonM, &easterMonD);
+	const bool isAfterEasterFri =
+		t->tm_mon + 1 > easterFriM ||
+		(t->tm_mon + 1 == easterFriM && t->tm_mday >= easterFriD);
+	const bool isBeforeEasterMon =
+		t->tm_mon + 1 < easterMonM ||
+		(t->tm_mon + 1 == easterMonM && t->tm_mday <= easterMonD);
+	if (isAfterEasterFri && isBeforeEasterMon)
+		return "bunny";
+	if (t->tm_mon + 1 == 11 &&
+		t->tm_mday ==
+			GetNthDayOfWeek(t->tm_year + 1900, 11, 4, 4)) // 4th Thu of Nov
+		return "capotain";
+	return NULL;
+}
+
 static direction_e GetLegDirAndFrame(
 	const TActor *a, const direction_e bodyDir, int *frame);
 static ActorPics GetUnorderedPics(
@@ -318,12 +401,19 @@ static ActorPics GetUnorderedPics(
 		pics.Head, BODY_PART_HEAD, c->Class->Sprites, anim, frame, dir,
 		GUNSTATE_READY);
 
+	const char *festiveHat = GetFestiveHat();
 	for (HeadPart hp = HEAD_PART_HAIR; hp < HEAD_PART_COUNT; hp++)
 	{
-		if (c->Class->HasHeadParts[hp])
+		// Holiday override
+		const bool hasHeadPart = c->Class->HasHeadParts[hp] ||
+								 (hp == HEAD_PART_HAT && festiveHat != NULL);
+		if (hasHeadPart)
 		{
+			const char *headPart = (hp == HEAD_PART_HAT && festiveHat)
+									   ? festiveHat
+									   : c->HeadParts[hp];
 			pics.HeadParts[hp] =
-				GetHeadPartPic(c->HeadParts[hp], hp, headDir, grimace, colors);
+				GetHeadPartPic(headPart, hp, headDir, grimace, colors);
 			pics.HeadPartOffsets[hp] = GetActorDrawOffset(
 				pics.HeadParts[hp], BODY_PART_HEAD, c->Class->Sprites, anim,
 				frame, dir, GUNSTATE_READY);
@@ -578,6 +668,10 @@ const Pic *GetHeadPartPic(
 	sprintf(buf, "chars/%s/%s", subpaths[hp], name);
 	const NamedSprites *ns =
 		PicManagerGetCharSprites(&gPicManager, buf, colors);
+	if (ns == NULL)
+	{
+		return NULL;
+	}
 	return CArrayGet(&ns->pics, idx);
 }
 static const Pic *GetBodyPic(
