@@ -22,7 +22,7 @@
 	This file incorporates work covered by the following copyright and
 	permission notice:
 
-	Copyright (c) 2013-2022 Cong Xu
+	Copyright (c) 2013-2023 Cong Xu
 	All rights reserved.
 
 	Redistribution and use in source and binary forms, with or without
@@ -272,7 +272,8 @@ void MapShowExitArea(Map *map, const int i)
 	const int bottom = top + exit->R.Size.y;
 
 	const TileClass *exitClass = TileClassesGetExit(
-		map->TileClasses, &gPicManager, gMission.missionData->ExitStyle, false);
+		map->TileClasses, &gPicManager, gMission.missionData->ExitStyle,
+		false);
 	const TileClass *exitShadowClass = TileClassesGetExit(
 		map->TileClasses, &gPicManager, gMission.missionData->ExitStyle, true);
 
@@ -338,15 +339,34 @@ bool MapPosIsInLockedRoom(const Map *map, const struct vec2 pos)
 	return MapTileIsInLockedRoom(map, tilePos);
 }
 
+bool MapTryPlaceDestroyObject(
+	MapBuilder *mb, const Mission *m, const int objective, const struct vec2i pos, const bool strict)
+{
+	const Objective *o = CArrayGet(&m->Objectives, objective);
+	// Pick a random map object out of the available ones
+	const int i = RAND_INT(0, (int)o->u.MapObjects.size - 1);
+	const MapObject *mo = *(const MapObject **)CArrayGet(&o->u.MapObjects, i);
+	return MapTryPlaceOneObject(mb, pos, mo, ObjectiveToThing(objective), strict);
+}
+
+void MapPlacePickup(
+	const PickupClass *p, const struct vec2 pos, const int flags)
+{
+	GameEvent e = GameEventNew(GAME_EVENT_ADD_PICKUP);
+	strcpy(e.u.AddPickup.PickupClass, p->Name);
+	e.u.AddPickup.ThingFlags = flags;
+	e.u.AddPickup.Pos = Vec2ToNet(pos);
+	GameEventsEnqueue(&gGameEvents, e);
+}
+
 void MapPlaceCollectible(
 	const Mission *m, const int objective, const struct vec2 pos)
 {
 	const Objective *o = CArrayGet(&m->Objectives, objective);
-	GameEvent e = GameEventNew(GAME_EVENT_ADD_PICKUP);
-	strcpy(e.u.AddPickup.PickupClass, o->u.Pickup->Name);
-	e.u.AddPickup.ThingFlags = ObjectiveToThing(objective);
-	e.u.AddPickup.Pos = Vec2ToNet(pos);
-	GameEventsEnqueue(&gGameEvents, e);
+	// Pick a random pickup out of the available ones
+	const int i = RAND_INT(0, (int)o->u.Pickups.size - 1);
+	const PickupClass *p = *(const PickupClass **)CArrayGet(&o->u.Pickups, i);
+	MapPlacePickup(p, pos, ObjectiveToThing(objective));
 }
 
 struct vec2 MapGenerateFreePosition(Map *map, const struct vec2i size)
