@@ -1,7 +1,7 @@
 /*
 	C-Dogs SDL
 	A port of the legendary (and fun) action/arcade cdogs.
-	Copyright (c) 2013-2019, 2021-2022 Cong Xu
+	Copyright (c) 2013-2019, 2021-2024 Cong Xu
 	All rights reserved.
 
 	Redistribution and use in source and binary forms, with or without
@@ -59,7 +59,8 @@ typedef enum
 	GUNTYPE_NORMAL,
 	GUNTYPE_MELEE,
 	GUNTYPE_GRENADE,
-	GUNTYPE_MULTI
+	GUNTYPE_MULTI,
+	GUNTYPE_COUNT
 } GunType;
 
 // Gun states
@@ -111,12 +112,14 @@ typedef struct
 	} u;
 	char *name;
 	char *Description;
+	char *Prerequisite; // WeaponClass
 	int Lock;
 	Mix_Chunk *SwitchSound;
 	bool CanDrop;	// whether this gun can be dropped to be picked up
 	char *DropGun;	// Gun to drop if an actor with this gun dies
 	bool IsRealGun; // whether this gun can be used as is by players
 	int Price;		// Price to purchase gun
+	int OverheatTicks;	// Number of ticks required to overheat the barrel and start smoking
 } WeaponClass;
 typedef struct
 {
@@ -126,6 +129,9 @@ typedef struct
 
 extern WeaponClasses gWeaponClasses;
 
+const char *GunTypeStr(const GunType t);
+void GunTypeGetSlotStartEnd(const GunType gt, int *start, int *end);
+
 void WeaponClassesInitialize(WeaponClasses *wcs);
 void WeaponClassesLoadJSON(WeaponClasses *wcs, CArray *classes, json_t *root);
 void WeaponClassesClear(CArray *classes);
@@ -157,6 +163,9 @@ const WeaponClass *WeaponClassGetBarrel(
 	const WeaponClass *wc, const int barrel);
 const BulletClass *WeaponClassGetBullet(
 	const WeaponClass *wc, const int barrel);
+const WeaponClass *WeaponClassGetPrerequisite(const WeaponClass *wc);
+bool WeaponClassesAreRelated(const WeaponClass *wc1, const WeaponClass *wc2);
+int WeaponClassFullPrice(const WeaponClass *wc);
 
 #define WC_BARREL_ATTR(_wc, _attr, _barrel)                                   \
 	((_wc).Type == GUNTYPE_MULTI                                              \
