@@ -22,7 +22,7 @@
 	This file incorporates work covered by the following copyright and
 	permission notice:
 
-	Copyright (c) 2013-2018, 2020-2022 Cong Xu
+	Copyright (c) 2013-2018, 2020-2024 Cong Xu
 	All rights reserved.
 
 	Redistribution and use in source and binary forms, with or without
@@ -66,6 +66,7 @@
 #include <cdogs/joystick.h>
 #include <cdogs/keyboard.h>
 #include <cdogs/log.h>
+#include <cdogs/map_wolf.h>
 #include <cdogs/music.h>
 #include <cdogs/net_client.h>
 #include <cdogs/net_server.h>
@@ -309,6 +310,7 @@ typedef struct
 	char suffixes[CDOGS_PATH_MAX];
 	char suffixnames[CDOGS_PATH_MAX];
 	EventWaitResult waitResult;
+	int endCounter;
 } PlayerSelectionData;
 static void PlayerSelectionTerminate(GameLoopData *data);
 static void PlayerSelectionOnExit(GameLoopData *data);
@@ -358,26 +360,12 @@ static void PlayerSelectionOnExit(GameLoopData *data)
 	PlayerSelectionData *pData = data->Data;
 	if (pData->waitResult == EVENT_WAIT_OK)
 	{
-		for (int i = 0, idx = 0; i < (int)gPlayerDatas.size; i++, idx++)
-		{
-			PlayerData *p = CArrayGet(&gPlayerDatas, i);
-			if (!p->IsLocal)
-			{
-				idx--;
-				continue;
-			}
-
-			// For any player slots not picked, turn them into AIs
-			if (p->inputDevice == INPUT_DEVICE_UNSET)
-			{
-				PlayerTrySetInputDevice(p, INPUT_DEVICE_AI, 0);
-			}
-		}
-
 		if (!gCampaign.IsClient)
 		{
 			gCampaign.MissionIndex = 0;
 		}
+		MapWolfN3DCheckAndLoadCustomQuiz(
+			gCampaign.Setting.Title, &gPlayerDatas);
 	}
 	else
 	{
@@ -407,34 +395,38 @@ static GameLoopResult PlayerSelectionUpdate(GameLoopData *data, LoopRunner *l)
 		return UPDATE_RESULT_OK;
 	}
 
-	// Menu input
 	int idx = 0;
-	const bool useMenuCmd = GetNumPlayers(PLAYER_ANY, true, true);
-	for (int i = 0; i < (int)gPlayerDatas.size; i++, idx++)
+	if (pData->endCounter == 0)
 	{
-		const PlayerData *p = CArrayGet(&gPlayerDatas, i);
-		if (!p->IsLocal)
+		// Menu input
+		const int localHumanPlayers = GetNumPlayers(PLAYER_ANY, true, true);
+		const bool useMenuCmd = (localHumanPlayers <= 1);
+		for (int i = 0; i < (int)gPlayerDatas.size; i++, idx++)
 		{
-			idx--;
-			continue;
-		}
-		if (p->inputDevice != INPUT_DEVICE_UNSET)
-		{
-			MenuSystem *ms = &pData->menus[idx].ms;
-			if (ms->current->customPostUpdateFunc)
+			const PlayerData *p = CArrayGet(&gPlayerDatas, i);
+			if (!p->IsLocal)
 			{
-				ms->current->customPostUpdateFunc(
-					ms->current, ms->current->customPostUpdateData);
+				idx--;
+				continue;
 			}
-			MenuUpdateMouse(ms);
-			if (useMenuCmd)
+			if (p->inputDevice != INPUT_DEVICE_UNSET)
 			{
-				cmds[idx] |=
-					GetMenuCmd(&gEventHandlers, ms->current->mouseHover);
-			}
-			if (!MenuIsExit(ms) && cmds[idx])
-			{
-				MenuProcessCmd(ms, cmds[idx]);
+				MenuSystem *ms = &pData->menus[idx].ms;
+				if (ms->current->customPostUpdateFunc)
+				{
+					ms->current->customPostUpdateFunc(
+						ms->current, ms->current->customPostUpdateData);
+				}
+				MenuUpdateMouse(ms);
+				if (useMenuCmd)
+				{
+					cmds[idx] |=
+						GetMenuCmd(&gEventHandlers, ms->current->mouseHover);
+				}
+				if (!MenuIsExit(ms) && cmds[idx])
+				{
+					MenuProcessCmd(ms, cmds[idx]);
+				}
 			}
 		}
 	}
@@ -465,16 +457,45 @@ static GameLoopResult PlayerSelectionUpdate(GameLoopData *data, LoopRunner *l)
 	}
 	if (isDone && hasAtLeastOneInput)
 	{
-		pData->waitResult = EVENT_WAIT_OK;
-		if (!gCampaign.IsClient && CanLevelSelect(gCampaign.Entry.Mode))
+		// For any player slots not picked, turn them into AIs
+		bool hasAIPlayers = false;
+		idx = 0;
+		for (int i = 0; i < (int)gPlayerDatas.size; i++, idx++)
 		{
-			LoopRunnerChange(l, LevelSelection(&gGraphicsDevice));
+			PlayerData *p = CArrayGet(&gPlayerDatas, i);
+			if (!p->IsLocal)
+			{
+				idx--;
+				continue;
+			}
+			if (p->inputDevice == INPUT_DEVICE_UNSET ||
+				p->inputDevice == INPUT_DEVICE_AI)
+			{
+				hasAIPlayers = true;
+				PlayerTrySetInputDevice(p, INPUT_DEVICE_AI, 0);
+				pData->menus[idx].ms.current =
+					MenuGetSubmenuByName(pData->menus[idx].ms.root, "Done");
+			}
+		}
+		if (hasAIPlayers && pData->endCounter < 70)
+		{
+			// If we have AI players, wait a bit before continuing
+			pData->endCounter++;
+			return UPDATE_RESULT_DRAW;
 		}
 		else
 		{
-			LoopRunnerChange(l, GameOptions(gCampaign.Entry.Mode));
+			pData->waitResult = EVENT_WAIT_OK;
+			if (!gCampaign.IsClient && CanLevelSelect(gCampaign.Entry.Mode))
+			{
+				LoopRunnerChange(l, LevelSelection(&gGraphicsDevice));
+			}
+			else
+			{
+				LoopRunnerChange(l, GameOptions(gCampaign.Entry.Mode));
+			}
+			return UPDATE_RESULT_OK;
 		}
-		return UPDATE_RESULT_OK;
 	}
 
 	AssignPlayerInputDevices(&gEventHandlers);
@@ -575,8 +596,14 @@ GameLoopData *GameOptions(const GameMode gm)
 		I("Game.Difficulty");
 		I("Game.EnemyDensity");
 		I("Game.NonPlayerHP");
-		I("Game.PlayerHP");
-		I("Game.Lives");
+		if (gCampaign.Setting.PlayerMaxHP == 0)
+		{
+			I("Game.PlayerHP");
+		}
+		if (gCampaign.Setting.Lives == 0)
+		{
+			I("Game.Lives");
+		}
 		I("Game.HealthPickups");
 		I("Game.RandomSeed");
 		MenuAddSubmenu(ms->root, MenuCreateSeparator(""));
@@ -592,7 +619,10 @@ GameLoopData *GameOptions(const GameMode gm)
 		I("StartServer");
 		break;
 	case GAME_MODE_DEATHMATCH:
-		I("Game.PlayerHP");
+		if (gCampaign.Setting.PlayerMaxHP == 0)
+		{
+			I("Game.PlayerHP");
+		}
 		I("Deathmatch.Lives");
 		I("Game.HealthPickups");
 		I("Game.RandomSeed");
@@ -675,57 +705,56 @@ static GameLoopResult GameOptionsUpdate(GameLoopData *data, LoopRunner *l)
 			MissionOptionsTerminate(&gMission);
 			CampaignUnload(&gCampaign);
 			LoopRunnerPop(l);
+			return UPDATE_RESULT_OK;
+		}
+		if (!ConfigApply(&gConfig, NULL))
+		{
+			LOG(LM_MAIN, LL_ERROR,
+				"Failed to apply config; reset to last used");
+			ConfigResetChanged(&gConfig);
 		}
 		else
 		{
-			if (!ConfigApply(&gConfig, NULL))
-			{
-				LOG(LM_MAIN, LL_ERROR,
-					"Failed to apply config; reset to last used");
-				ConfigResetChanged(&gConfig);
-			}
-			else
-			{
-				// Save options for later
-				ConfigSave(&gConfig, GetConfigFilePath(CONFIG_FILE));
-			}
+			// Save options for later
+			ConfigSave(&gConfig, GetConfigFilePath(CONFIG_FILE));
+		}
 
-			// Set allowed weapons
-			// First check if the player has unwittingly disabled all weapons
-			// if so, enable all weapons
-			bool allDisabled = true;
+		// Set allowed weapons
+		// First check if the player has unwittingly disabled all weapons
+		// if so, enable all weapons
+		bool allDisabled = true;
+		for (int i = 0, j = 0; i < (int)gData->allowed.size; i++, j++)
+		{
+			const bool *allowed = CArrayGet(&gData->allowed, i);
+			if (*allowed)
+			{
+				allDisabled = false;
+				break;
+			}
+		}
+		if (!allDisabled)
+		{
 			for (int i = 0, j = 0; i < (int)gData->allowed.size; i++, j++)
 			{
 				const bool *allowed = CArrayGet(&gData->allowed, i);
-				if (*allowed)
+				if (!*allowed)
 				{
-					allDisabled = false;
-					break;
+					CArrayDelete(&gMission.Weapons, j);
+					j--;
 				}
 			}
-			if (!allDisabled)
-			{
-				for (int i = 0, j = 0; i < (int)gData->allowed.size; i++, j++)
-				{
-					const bool *allowed = CArrayGet(&gData->allowed, i);
-					if (!*allowed)
-					{
-						CArrayDelete(&gMission.Weapons, j);
-						j--;
-					}
-				}
-			}
-
-			gCampaign.OptionsSet = true;
-
-			// If enabled, start net server
-			if (!gCampaign.IsClient && ConfigGetBool(&gConfig, "StartServer"))
-			{
-				NetServerOpen(&gNetServer);
-			}
-			LoopRunnerPush(
-				l, ScreenMissionBriefing(&gCampaign.Setting, &gMission));
 		}
+
+		gCampaign.OptionsSet = true;
+
+		// If enabled, start net server
+		if (!gCampaign.IsClient && ConfigGetBool(&gConfig, "StartServer"))
+		{
+			NetServerOpen(
+				&gNetServer, (uint16_t)ConfigGetInt(&gConfig, "ListenPort"));
+		}
+		LoopRunnerPush(
+			l, ScreenMissionBriefing(&gCampaign.Setting, &gMission));
 		return UPDATE_RESULT_OK;
 	}
 	return UPDATE_RESULT_DRAW;
@@ -786,13 +815,11 @@ bail:
 	if (!gCampaign.IsLoaded)
 	{
 		LoopRunnerPop(l);
+		return UPDATE_RESULT_OK;
 	}
-	else
-	{
-		LoopRunnerPush(
-			l, ScreenLoading(
-				   "Starting game...", true,
-				   RunGame(&gCampaign, &gMission, &gMap), true));
-	}
+	LoopRunnerPush(
+		l, ScreenLoading(
+			   "Starting game...", true, RunGame(&gCampaign, &gMission, &gMap),
+			   true));
 	return UPDATE_RESULT_OK;
 }
